
/**
 * (c) Copyright 1993, 1994, 1995, 1996 Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED
 * Permission to use, copy, modify, and distribute this software for
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission.
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * US Government Users Restricted Rights
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(TM) is a trademark of Silicon Graphics, Inc.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "gl.h"

#include "glu.h"
#include "agl.h"
#include "tk.h"
#include "aux.h"
#include "atlantis.h"

fishRec sharks[NUM_SHARKS];
fishRec momWhale;
fishRec babyWhale;
fishRec dolph;

GLboolean Timing = GL_TRUE;

GLint w_win = 400;
GLint h_win = 400;

static float sin_table[361];

GLint count  = 0;

static double filter(double in, double *save)
{
	static double k1 = 0.9;
	static double k2 = 0.05;

	save[3] = in;
	save[1] = save[0]*k1 + k2*(save[3] + save[2]);

	save[0]=save[1];
	save[2]=save[3];

	return(save[1]);
}

void DrawStr(char *str)
{
	static int bitmapBase = 0;
	
	if(bitmapBase == 0)
	{
		bitmapBase = glGenLists(256);
		aglUseFont(aglGetCurrentContext(), kFontIDNewYork, normal, 10, 0, 256, bitmapBase);
	}
	
	glListBase(bitmapBase);
	glCallLists(strlen(str), GL_UNSIGNED_BYTE, (GLubyte *) str);
}

float sin_func(int d)
{
	while(d >  180) d -= 360;
	while(d < -180) d += 360;
	
	return sin_table[d + 180];
}

void InitFishs(void)
{
    int i;

    for (i = 0; i < NUM_SHARKS; i++) {
        sharks[i].x = 70000.0 + rand() % 6000;
        sharks[i].y = rand() % 6000;
        sharks[i].z = rand() % 6000;
        sharks[i].psi = rand() % 360 - 180.0;
        sharks[i].v = 1.0;
    }

    dolph.x = 30000.0;
    dolph.y = 0.0;
    dolph.z = 6000.0;
    dolph.psi = 90.0;
    dolph.theta = 0.0;
    dolph.v = 3.0;

    momWhale.x = 70000.0;
    momWhale.y = 0.0;
    momWhale.z = 0.0;
    momWhale.psi = 90.0;
    momWhale.theta = 0.0;
    momWhale.v = 3.0;

    babyWhale.x = 60000.0;
    babyWhale.y = -2000.0;
    babyWhale.z = -2000.0;
    babyWhale.psi = 90.0;
    babyWhale.theta = 0.0;
    babyWhale.v = 3.0;
}

void Init(void)
{
	 static float ambient[] = {0.2, 0.2, 0.2, 1.0};
    static float diffuse[] = {1.0, 1.0, 1.0, 1.0};
    static float position[] = {0.0, 1.0, 0.0, 0.0};
    static float mat_shininess[] = {90.0};
    static float mat_specular[] = {0.8, 0.8, 0.8, 1.0};
    static float mat_diffuse[] = {0.46, 0.66, 0.795, 1.0};
    static float mat_ambient[] = {0.3, 0.4, 0.5, 1.0};
    static float lmodel_ambient[] = {0.4, 0.4, 0.4, 1.0};
    static float lmodel_localviewer[] = {0.0};
    GLfloat map1[4] = {0.0, 0.0, 0.0, 0.0};
    GLfloat map2[4] = {0.0, 0.0, 0.0, 0.0};
    static float fog_color[] = {0.0, 0.5, 0.9, 1.0};
    int i;
    
    srand((GLuint) tkNow() % 10000);
    
    for(i = 0; i <= 360; i++)
    {
    	sin_table[i] = sin((double) (i - 180) * RRAD);
    }

    glFrontFace(GL_CCW);
 
    glDepthFunc(GL_LEQUAL);
    glEnable(GL_DEPTH_TEST);

    glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
    glLightfv(GL_LIGHT0, GL_POSITION, position);
    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
    glLightModelfv(GL_LIGHT_MODEL_LOCAL_VIEWER, lmodel_localviewer);
    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);

    glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, mat_shininess);
    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular);
    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mat_diffuse);
    glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, mat_ambient);

    InitFishs();

    glEnable(GL_FOG);
	 glFogi(GL_FOG_MODE, GL_EXP);
	 glFogf(GL_FOG_DENSITY, 0.0000025);
	 glFogfv(GL_FOG_COLOR, fog_color);

    glClearColor(0.0, 0.5, 0.9, 1.0);
    
    //glEnable(GL_SCISSOR_TEST);
    
    //aglEnable(aglGetCurrentContext(), AGL_BUFFER_RECT);
    //aglEnable(aglGetCurrentContext(), AGL_SWAP_RECT);
}

void Reshape(int width, int height)
{
	GLint r[4];
	
    glViewport(0, 0, width, height);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(60.0, (GLfloat) width / (GLfloat) height, 2000.0, 300000.0);
    glMatrixMode(GL_MODELVIEW);
    
    w_win = width;
    h_win = height;
    
    r[0] = 40;
    r[1] = 40;
    r[2] = width - 80;
    r[3] = height - 80;
    
    aglSetInteger(aglGetCurrentContext(), AGL_BUFFER_RECT, r);
    
    r[2] -= 80;
    r[3] -= 80;
    
    aglSetInteger(aglGetCurrentContext(), AGL_SWAP_RECT, r);
    
    r[0] += 40;
    r[1] += 40;
    r[2] -= 80;
    r[3] -= 80;
    
    glScissor(r[0], r[1], r[2], r[3]);
}

void Animate(void)
{
    int i;

    for (i = 0; i < NUM_SHARKS; i++) 
    {
        SharkPilot(&sharks[i]);
        SharkMiss(i);
    }
    
    WhalePilot(&dolph);
    dolph.phi++;
    
    WhalePilot(&momWhale);
    momWhale.phi++;
    
    WhalePilot(&babyWhale);
    babyWhale.phi++;
}

static void QuitKey(void)
{
	exit(1);
}

static void TimingKey(void)
{
	Timing = !Timing;
}

void Display(void)
{
    int i;
    static double th[4] = {0.0, 0.0, 0.0, 0.0};
	static double t1 = 0.0, t2 = 0.0, t;
	char num_str[200];
    
    t1 = t2;
    
    glDisable(GL_DITHER);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glEnable(GL_DITHER);

    for (i = 0; i < NUM_SHARKS; i++) {
        glPushMatrix();
        FishTransform(&sharks[i]);
        DrawShark(&sharks[i]);
        glPopMatrix();
    }
    
    glPushMatrix();
    FishTransform(&dolph);
    DrawDolphin(&dolph);
    glPopMatrix();

    glPushMatrix();
    FishTransform(&momWhale);
    DrawWhale(&momWhale);
    glPopMatrix();

    glPushMatrix();
    FishTransform(&babyWhale);
    glScalef(0.45, 0.45, 0.3);
    DrawWhale(&babyWhale);
    glPopMatrix();
    
    if(Timing)
    {
		t2 = tkNow();
		t = t2 - t1;
		if(t > 0.0001) t = 1.0 / t;
		
		glDisable(GL_LIGHTING);
		//glDisable(GL_DEPTH_TEST);
		
		glColor3f(1.0, 0.0, 0.0);
		
		glMatrixMode (GL_PROJECTION);
		glPushMatrix();
		glLoadIdentity();
		glOrtho(0, w_win, 0, h_win, -10.0, 10.0);
		
		glRasterPos2f(5.0, 5.0);
		
		sprintf(num_str, "%0.2f Hz, %dx%d, Renderer: %s", filter(t, th), w_win, h_win, glGetString(GL_RENDERER));
		DrawStr(num_str);
		
		glPopMatrix();
		glMatrixMode(GL_MODELVIEW);
		
		glEnable(GL_LIGHTING);
		//glEnable(GL_DEPTH_TEST);
	}
	
    count++;

    auxSwapBuffers();
}

void main(int argc, char **argv)
{
  	//auxInitDisplayMode (AUX_DOUBLE | AUX_RGB | AUX_DEPTH | AUX_FULLSCREEN);
  	//auxInitPosition (0, 0, 640, 480);
  	
  	auxInitDisplayMode (AUX_DOUBLE | AUX_RGB | AUX_DEPTH);
    auxInitPosition (30, 60, w_win, h_win);
    
    auxInitWindow ("Atlantis");
    
    Init();
    
    auxKeyFunc(AUX_ESCAPE, QuitKey);
    auxKeyFunc(AUX_t, TimingKey);
    auxReshapeFunc(Reshape);
    auxIdleFunc(Animate);
    auxMainLoop(Display);
}
